@@ -1,129 +1,146 @@
-#' Initial covariance parameters (ICP)
+#' Guess the parameters of a spatial covariance function
+#'
+#' @description
+#' Guess the parameters of the spatial covariance function of a random, regionalized variable. A
+#' guess of such parameters is required to start the fitting functions of many geostatistical
+#' packages such as __gstat__, __geoR__, and __georob__.
+#'
+#' @inheritParams variogramBins
+#'
+#' @param z Numeric vector with the values of the regionalized variable for which the values for the
+#' spatial covariance parameters should be guessed.
+#'
+#' @param lags Numeric scalar defining the width of the variogram bins or a numeric vector with the
+#' lower and upper bounds of the variogram bins. If missing, the variogram bins are computed using
+#' [pedometrics::variogramBins()]. See \sQuote{Details} for more information.
+#'
+#' @param method Character keyword defining the method used for guessing the spatial covariance
+#' parameters of the regionalized variable. Defaults to `method = "a"`. See \sQuote{Details} for
+#' more information.
+#'
+#' @param min.npairs Positive integer defining the minimum number of point-pairs required so that a
+#' variogram bin is used to guessing the spatial covariance parameters of the  of the regionalized
+#' variable. Defaults to `min.npairs = 30`.
+#'
+#' @param model Character keyword defining the spatial covariance function that will be fitted to
+#' the data of the regionalized variable. Currently, most of the basic spatial covariance function
+#' are accepted. See `geoR::cov.spatial()` for more information. Defaults to `model = "matern"`.
+#'
+#' @param nu Numerical value for the additional smoothness parameter \eqn{\nu} of the spatial
+#' covariance function of the regionalized variable. See `RandomFields::RMmodel()` and argument
+#' `kappa` of `geoR::cov.spatial()` for more information.
+#'
+#' @param plotit Should the guessed spatial covariance parameters be plotted along with the sample
+#' (experimental) variogram of the regionalized variable? Defaults to `plotit = FALSE`.
+#'
+#' @param estimator Character keyword defining the estimator for computing the sample (experimental)
+#' variogram of the regionalized variable, with options `"qn"` (default), `"mad"`, `"matheron"`, and
+#' `"ch"`. See `georob::sample.variogram()` for more details.
+#'
+#' @return
+#' A vector of numerical values, the guesses for the spatial covariance parameters of the
+#' regionalized variable:
 #' 
-#' Guess the initial values for the covariance parameters required to fit a variogram model.
-#' 
-#' @inheritParams vgmLags
-#' 
-#' @param z Numeric vector with the values of the response variable for which the initial values for the
-#' covariance parameters should be guessed.
-#' 
-#' @param lags Numeric scalar defining the width of the lag-distance classes, or a numeric vector with the 
-#' lower and upper bounds of the lag-distance classes. If missing, the lag-distance classes are computed using 
-#' \code{\link[pedometrics]{vgmLags}}. See \sQuote{Details} for more information.
-#' 
-#' @param method Character keyword defining the method used for guessing the initial covariance parameters. 
-#' Defaults to \code{method = "a"}. See \sQuote{Details} for more information.
-#' 
-#' @param min.npairs Positive integer defining the minimum number of point-pairs required so that a 
-#' lag-distance class is used for guessing the initial covariance parameters. Defaults to `min.npairs = 30`.
-#' 
-#' @param model Character keyword defining the variogram model that will be fitted to the data. Currently, 
-#' most basic variogram models are accepted. See \code{\link[geoR]{cov.spatial}} for more information. Defaults
-#' to `model = "matern"`.
-#' 
-#' @param nu numerical value for the additional smoothness parameter \eqn{\nu} of the correlation function. See
-#' \code{\link[RandomFields]{RMmodel}} and argument \code{kappa} of \code{\link[geoR]{cov.spatial}} for more 
-#' information.
-#' 
-#' @param plotit Should the guessed initial covariance parameters be plotted along with the sample variogram? 
-#' Defaults to \code{plotit = FALSE}.
-#' 
-#' @param estimator Character keyword defining the estimator for computing the sample variogram, with options 
-#' `"qn"`, `"mad"`, `"matheron"`, and `"ch"`. Defaults to `estimator = "qn"`. See 
-#' \code{\link[georob]{sample.variogram}} for more details.
-#' 
-#' @return A vector of numeric values: the guesses for the covariance parameters nugget, partial sill, and 
-#' range.
-#' 
-#' @details There are five methods two guess the initial covariance parameters  (ICP). Two of them, `"a"` and
-#' `"c"`, rely a sample variogram with exponentially spaced lag-distance classes, while the other three, `"b"`,
-#' `"d"`, and `"e"`, use equidistant lag-distance classes (see \code{\link[pedometrics]{vgmLags}}). All of them
-#' are \href{https://en.wikipedia.org/wiki/Heuristic}{heuristic}.
-#' 
-#' Method `"a"` was developed in-house and is the most elaborated of them, specially for guessing the nugget
-#' variance.
-#' 
-#' Method `"b"` was proposed by \href{http://dx.doi.org/10.1016/0098-3004(95)00095-X}{Jian et al. (1996)} and
+#'   * nugget
+#'   * partial sill
+#'   * range
+#'
+#' @details
+#' There are five methods two guess the covariance parameters. Two of them, `"a"` and `"c"`, rely on
+#' a sample variogram with exponentially growing variogram bins, while the other three, `"b"`,
+#' `"d"`, and `"e"`, use equal-width variogram bins (see [pedometrics::variogramBins()]). All of
+#' them are [heuristic](https://en.wikipedia.org/wiki/Heuristic).
+#'
+#' Method `"a"` was developed in-house and is the most elaborated of them, specially for guessing
+#' the nugget variance.
+#'
+#' Method `"b"` was proposed by \doi{10.1016/0098-3004(95)00095-X}{Jian et al. (1996)} and
 #' is implemented in \href{https://support.sas.com/documentation/cdl/en/statug/63347/HTML/default/viewer.htm#statug_variogram_a0000000593.htm}{SAS/STAT(R) 9.22}.
+#'
+#' Method `"c"` is implemented in the __automap__-package and was developed by
+#' \doi{10.1016/j.cageo.2008.10.011}{Hiemstra et al. (2009)}.
+#'
+#' Method `"d"` was developed by \doi{10.1007/s11004-012-9434-1}{Desassis & Renard (2012)}.
+#'
+#' Method `"e"` was proposed by Larrondo et al. (2003)
+#' <http://www.ccgalberta.com/ccgresources/report05/2003-122-varfit.pdf> and is implemented in the
+#' VARFIT module of GSLIB <http://www.gslib.com/>.
+#'
+#' @references
+#' Desassis, N. & Renard, D. Automatic variogram modelling by iterative least squares: univariate
+#' and multivariate cases. _Mathematical Geosciences_. Springer Science + Business Media, v. 45, p.
+#' 453-470, 2012.
 #' 
-#' Method `"c"` is implemented in the __automap__-package and was developed by 
-#' \href{http://dx.doi.org/10.1016/j.cageo.2008.10.011}{Hiemstra et al. (2009)}.
+#' Hiemstra, P. H.; Pebesma, E. J.; Twenhöfel, C. J. & Heuvelink, G. B. Real-time automatic
+#' interpolation of ambient gamma dose rates from the Dutch radioactivity monitoring network.
+#' _Computers & Geosciences_. Elsevier BV, v. 35, p. 1711-1721, 2009.
 #' 
-#' Method `"d"` was developed by \href{http://dx.doi.org/10.1007/s11004-012-9434-1}{Desassis & Renard (2012)}.
+#' Jian, X.; Olea, R. A. & Yu, Y.-S. Semivariogram modelling by weighted least squares. _Computers &
+#' Geosciences_. Elsevier BV, v. 22, p. 387-397, 1996.
 #' 
-#' Method `"e"` was proposed by \href{http://www.ccgalberta.com/ccgresources/report05/2003-122-varfit.pdf}{Larrondo et al. (2003)} and is implemented in the VARFIT module of \href{http://www.gslib.com/}{GSLIB}.
+#' Larrondo, P. F.; Neufeld, C. T. & Deutsch, C. V. _VARFIT: a program for semi-automatic variogram
+#' modelling_. Edmonton: Department of Civil and Environmental Engineering, University of Alberta,
+#' p. 17, 2003.
 #' 
-#' @references 
+# @note 
+# Package __geoR__ is used to guess the range (scale) parameter of the following spatial covariance
+# functions: "matern" (except when `nu = 0.5`), "powered.exponential", "stable", "cauchy",
+# "gencauchy", "gneiting", and "gneiting.matern". The practical range is the distance at which the
+# semivariance reaches its maximum, i.e. the sill.
 #' 
-#' Desassis, N. & Renard, D. Automatic variogram modelling by iterative least squares: univariate and 
-#' multivariate cases. _Mathematical Geosciences_. Springer Science \eqn{+} Business Media, v. 45, p. 453-470,
-#' 2012.
+#' @section Dependencies:
+# The __geoR__ package, provider of functions for the analysis of geostatistical data in R, is
+# required for [pedometrics::variogramGuess()] to work. The development version of the __geoR__
+# package is available on <http://www.leg.ufpr.br/geoR/> while its old versions are available on
+# the CRAN archive at <https://cran.r-project.org/src/contrib/Archive/geoR/>.
 #' 
-#' Hiemstra, P. H.; Pebesma, E. J.; Twenhöfel, C. J. & Heuvelink, G. B. Real-time automatic interpolation of
-#' ambient gamma dose rates from the Dutch radioactivity monitoring network. _Computers & Geosciences_. 
-#' Elsevier BV, v. 35, p. 1711-1721, 2009.
-#' 
-#' Jian, X.; Olea, R. A. & Yu, Y.-S. Semivariogram modelling by weighted least squares. 
-#' _Computers & Geosciences_. Elsevier BV, v. 22, p. 387-397, 1996.
-#' 
-#' Larrondo, P. F.; Neufeld, C. T. & Deutsch, C. V. _VARFIT: a program for semi-automatic variogram modelling_.
-#' Edmonton: Department of Civil and Environmental Engineering, University of Alberta, p. 17, 2003.
-#' 
-#' @note 
-#' Package __geoR__ is used to guess the range (scale) parameter of the following covariance models: "matern" (except when `nu = 0.5`), "powered.exponential", "stable", "cauchy", "gencauchy", "gneiting", and "gneiting.matern". However, __geoR__ is an orphan package since 2020-01-12. Thus, if __geoR__ is not installed, a guess of the practical range of these covariance models is returned. The practical range is the distance at which the semivariance reaches its maximum, i.e. the sill.
+#' The __georob__ package, provider of functions for the robust geostatistical analysis of spatial
+#' data in R, is required for [pedometrics::variogramGuess()] to work. The old versions of the
+#' __georob__ package are available on the CRAN archive at
+#' <https://cran.r-project.org/src/contrib/Archive/georob/>.
 #' 
 #' @author Alessandro Samuel-Rosa \email{alessandrosamuelrosa@@gmail.com}
 #' 
-#' @seealso \code{\link[pedometrics]{vgmLags}}, 
-#'          \code{\link[georob]{sample.variogram}}, 
-#'          \code{\link[automap]{autofitVariogram}}
-#' 
-#' @concept variogram
-#' @export
+#' @seealso [pedometrics::variogramBins()]
 #' 
 #' @examples
-#' data(meuse, package = "sp")
-#' icp <- vgmICP(z = log(meuse$copper), coords = meuse[, 1:2])
-# FUNCTION - MAIN #############################################################################################
-vgmICP <- 
-  function (z, coords, lags, cutoff = 0.5, method = "a", min.npairs = 30, model = "matern", nu = 0.5, 
-            estimator = "qn", plotit = FALSE) {
-    
-    # Check arguments
+# if (all(c(require(sp), require(georob), require(geoR)))) {
+#' if (all(c(require(sp), require(georob)))) {
+#'   data(meuse, package = "sp")
+#'   icp <- variogramGuess(z = log(meuse$copper), coords = meuse[, 1:2])
+#' }
+#' @concept variogram
+# FUNCTION - MAIN ##################################################################################
+#' @export
+#' @rdname variogramGuess
+variogramGuess <-
+  function(z, coords, lags, cutoff = 0.5, method = "a", min.npairs = 30, model = "matern", nu = 0.5,
+          estimator = "qn", plotit = FALSE) {
+    # check if suggested packages are installed
+    if (!requireNamespace("georob")) stop("georob package is missing")
+    # if (!requireNamespace("geoR")) stop("geoR package is missing") # MOVED TO 
+    # check function arguments
     cov_models <- c(
-      "matern", "exponential", "gaussian", "spherical", "circular", "cubic", "wave", "linear", "power", 
-      "powered.exponential", "stable", "cauchy", "gencauchy", "gneiting", "gneiting.matern", "pure.nugget")
-    
+      "matern", "exponential", "gaussian", "spherical", "circular", "cubic", "wave", "linear",
+      "power", "powered.exponential", "stable", "cauchy", "gencauchy", "gneiting",
+      "gneiting.matern", "pure.nugget")
     if (!model %in% cov_models) {
-      stop (paste("model '", model, "' is not implemented", sep = ""))
+      stop(paste("model '", model, "' is not implemented", sep = ""))
     }
-    
-    # Check if suggested packages are installed
-    # geoR has been orphaned on 2020-01-12
-    # pkg <- c("georob", "geoR")
-    pkg <- c("georob")
-    id <- !sapply(pkg, requireNamespace, quietly = TRUE)
-    if (any(id)) {
-      pkg <- paste(pkg[which(id)], collapse = " ")
-      stop(paste("Package(s) needed for this function to work but not installed:", pkg), call. = FALSE)
-    }
-    
     # Check lags and max.dist
     if (missing(lags)) {
       if (method %in% c("a", "c")) {
-        lags <- vgmLags(coords, cutoff = cutoff)
+        lags <- variogramBins(coords, cutoff = cutoff)
       } else {
-        lags <- vgmLags(coords, n.lags = 15, type = "equi", cutoff = cutoff)
+        lags <- variogramBins(coords, n.lags = 15, type = "equi", cutoff = cutoff)
       }
     }
     cutoff <- sqrt(
       sum(apply(apply(coords[, 1:2], 2, range), 2, diff) ^ 2)) * cutoff
-    
     # Compute variogram
     v <- georob::sample.variogram(
       object = z, locations = coords, lag.dist.def = lags, max.lag = cutoff, estimator = estimator)
     lags0 <- length(v$lag.dist)
-    
     # Merge lag-distance classes that have too few point-pairs
     if (any(v$npairs < min.npairs)) {
       message("correcting lags for minimum number of point-pairs...")
@@ -131,15 +148,15 @@ vgmICP <-
       while (length(idx) >= 1) {
         lags <- lags[-idx[1]]
         v <- georob::sample.variogram(
-          object = z, locations = coords, lag.dist.def = lags, max.lag = cutoff, estimator = estimator)
+          object = z, locations = coords, lag.dist.def = lags, max.lag = cutoff,
+          estimator = estimator)
         idx <- which(v$npairs < min.npairs) + 1
       }
     }
     if (plotit) {
       graphics::plot(v)
     }
-    
-    # SILL ----------------------------------------------------------------------------------------------------
+    # SILL -----------------------------------------------------------------------------------------
     # The initial guess for the sill should be the easiest among all
     # parameters needed to fit a covariance model. Several rules are used in the
     # literature:
@@ -174,8 +191,7 @@ vgmICP <-
     if (plotit) {
       graphics::abline(h = sill, lty = "dashed")
     }
-    
-    # RANGE ---------------------------------------------------------------------------------------------------
+    # RANGE ----------------------------------------------------------------------------------------
     # In general, the initial guess for the range (scale) parameter is made based on the lag-distance classes
     # and on the dimensions of the study area. A common rule is to compute the initial range as half the 
     # maximum distance up to which lag-distance classes have been defined (Jian et al., 1996; Larrondo et al.,
@@ -191,13 +207,14 @@ vgmICP <-
     # semivariance is closest to the variance. The separation distance at the centre of this lag-distance class
     # is recorded. This value can be taken to be approximately equivalent to the practical range. Thus, it is
     # corrected using geoR::practicalRange.
+    # geoR IS ARCHIVED FROM TIME TO TIME... IT MUST BE REMOVED FRO HERE!!!
     range <- switch(
-      method, 
+      method,
       a = { # Samuel-Rosa2015
         v$lag.dist[which.min(abs(v$gamma[-c(1, length(v$gamma))] - sill)) + 1]
-      }, 
+      },
       b = { # JianEtAl1996
-        lags[length(lags)] * 0.5 
+        lags[length(lags)] * 0.5
       },
       c = { # HiemstraEtAl2009
         sqrt(sum(apply(apply(coords, 2, range), 2, diff) ^ 2)) * 0.1
@@ -210,7 +227,6 @@ vgmICP <-
       }
     )
     # Correct initial guess of the range parameter
-    # geoR has been orphaned on 2020-01-12
     if (model %in% c('linear', 'power')) {
       range <- Inf
     } else if (model == 'pure.nugget') {
@@ -225,21 +241,20 @@ vgmICP <-
       range <- range / 2.991457
     } else {
       # "matern", "powered.exponential", "stable", "cauchy", "gencauchy", "gneiting", "gneiting.matern"
-      if (requireNamespace('geoR', quietly = TRUE)) {
-        range <- range / geoR::practicalRange(cov.model = model, phi = 1, kappa = nu)
-      } else {
-        message(
-          paste('geoR not installed... returning guess of practical range for model', model)
-        )
+      # geoR is archived from time to time...
+      # if (requireNamespace("geoR", quietly = TRUE)) {
+      #   range <- range / geoR::practicalRange(cov.model = model, phi = 1, kappa = nu)
+      # } else {
+      #   message(
+      #     paste("geoR not installed... returning guess of practical range for model ", model)
+      #   )
         range <- range
-      }
+      # }
     }
-    
     if (plotit) {
       graphics::abline(v = range, lty = "dashed")
     }
-    
-    # NUGGET --------------------------------------------------------------------------------------------------
+    # NUGGET ---------------------------------------------------------------------------------------
     # The initial guess for the nugget variance is commonly made using one of
     # the following rules:
     # 1) use the minimum semivariance in the sample variogram (Hiemstra et al.,
@@ -253,7 +268,6 @@ vgmICP <-
     nugget <- switch(
       method,
       a = { # Samuel-Rosa
-        
         # Is the minimum gamma in lags others than the first?
         # 
         if (which.min(v$gamma) > 1) {
@@ -315,7 +329,6 @@ vgmICP <-
           # 
           # }
           # min(v$gamma)
-          
         } else {
           # The first lag-distance class holds the minimum gamma!
           # 
@@ -346,7 +359,6 @@ vgmICP <-
             if (!identical(order(v$gamma[1:3]), 1:3)) {
               # mean(c(v$gamma[1], min(v$gamma[-1])))
               mean(v$gamma[c(1, 3)])
-              
             } else {
               # Gamma increases monotonically from the first to the third lag,
               # but gamma at the first lag is too low. It may be better to use 
@@ -355,7 +367,6 @@ vgmICP <-
               # 
               # mean(v$gamma[c(1, 2)])
               mean(v$gamma[c(1, 1, 2)])
-              
             }
           } else {
             # Gamma in the second lag is closer to gamma in the first lag than
@@ -405,7 +416,6 @@ vgmICP <-
     if (plotit) {
       graphics::abline(h = nugget, lty = "dashed")
     }
-    
     # Guess the partial sill for a pure nugget effect model
     if (nugget < sill) {
       p_sill <- sill - nugget
@@ -413,8 +423,10 @@ vgmICP <-
       # p_sill <- 1e-12
       p_sill <- nugget * 1e-3
     }
-    
     # Prepare output
     res <- c(range = range, p_sill = p_sill, nugget = nugget)
-    return (res)
+    return(res)
   }
+#' @export
+#' @rdname variogramGuess
+vgmICP <- variogramGuess
